@@ -10,6 +10,7 @@ from __future__ import annotations
 
 import ast
 import inspect
+from collections import defaultdict
 from contextlib import suppress
 from itertools import zip_longest
 from pathlib import Path
@@ -43,9 +44,22 @@ from griffe.dataclasses import (
     Parameters,
 )
 from griffe.docstrings.parsers import Parser
-from griffe.exceptions import LastNodeError
+from griffe.exceptions import LastNodeError, NameResolutionError
 from griffe.expressions import Expression, Name
 
+builtin_decorators = {
+    "property",
+    "staticmethod",
+    "classmethod",
+}
+
+stdlib_decorators = {
+    "abc.abstractmethod",
+    "functools.cache",
+    "functools.cached_property",
+    "functools.lru_cache",
+}
+
 
 def visit(
     module_name: str,
@@ -126,11 +140,12 @@ class Visitor(BaseVisitor):  # noqa: WPS338
         self.extensions: Extensions = extensions.attach_visitor(self)
         self.parent: Module | None = parent
         self.current: Module | Class = None  # type: ignore[assignment]
-        self.in_decorator: bool = False
         self.docstring_parser: Parser | None = docstring_parser
         self.docstring_options: dict[str, Any] = docstring_options or {}
         self.lines_collection: LinesCollection = lines_collection or LinesCollection()
         self.modules_collection: ModulesCollection = modules_collection or ModulesCollection()
+        self.type_guarded: bool = False
+        self.overloads: dict[str, list[Function]] = defaultdict(list)
 
     def _get_docstring(self, node: ast.AST, strict: bool = False) -> Docstring | None:
         value, lineno, endlineno = get_docstring(node, strict=strict)
@@ -209,11 +224,14 @@ class Visitor(BaseVisitor):  # noqa: WPS338
         decorators = []
         if node.decorator_list:
             lineno = node.decorator_list[0].lineno
-            self.in_decorator = True
             for decorator_node in node.decorator_list:
-                decorators.append(Decorator(decorator_node.lineno, decorator_node.end_lineno))  # type: ignore[attr-defined]
-                self.visit(decorator_node)
-            self.in_decorator = False
+                decorators.append(
+                    Decorator(
+                        get_value(decorator_node),
+                        lineno=decorator_node.lineno,
+                        endlineno=decorator_node.end_lineno,  # type: ignore[attr-defined]
+                    )
+                )
         else:
             lineno = node.lineno
 
@@ -230,12 +248,62 @@ class Visitor(BaseVisitor):  # noqa: WPS338
             docstring=self._get_docstring(node),
             decorators=decorators,
             bases=bases,  # type: ignore[arg-type]
+            runtime=not self.type_guarded,
         )
         self.current[node.name] = class_
         self.current = class_
         self.generic_visit(node)
         self.current = self.current.parent  # type: ignore[assignment]
 
+    def decorators_to_labels(self, decorators: list[Decorator]) -> set[str]:  # noqa: WPS231
+        """Build and return a set of labels based on decorators.
+
+        Parameters:
+            decorators: The decorators to check.
+
+        Returns:
+            A set of labels.
+        """
+        labels = set()
+        for decorator in decorators:
+            decorator_value = decorator.value.split("(", 1)[0]
+            if decorator_value in builtin_decorators:
+                labels.add(decorator_value)
+            else:
+                names = decorator_value.split(".")
+                with suppress(NameResolutionError):
+                    resolved_first = self.current.resolve(names[0])
+                    resolved_name = ".".join([resolved_first, *names[1:]])
+                    if resolved_name in stdlib_decorators:
+                        if "abstract" in resolved_name:
+                            labels.add("abstract")
+                        elif "cache" in resolved_name:
+                            labels.add("cached")
+        return labels
+
+    def get_base_property(self, decorators: list[Decorator]) -> tuple[Function | None, str | None]:
+        """Check decorators to return the base property in case of setters and deleters.
+
+        Parameters:
+            decorators: The decorators to check.
+
+        Returns:
+            base_property: The property for which the setter/deleted is set.
+            property_function: Either `"setter"` or `"deleter"`.
+        """
+        for decorator in decorators:
+            names = decorator.value.split(".")
+            with suppress(ValueError):
+                base_name, base_function = names
+                property_setter_or_deleter = (
+                    base_function in {"setter", "deleter"}
+                    and base_name in self.current.members
+                    and self.current[base_name].has_labels({"property"})
+                )
+                if property_setter_or_deleter:
+                    return self.current[base_name], base_function
+        return None, None
+
     def handle_function(self, node: ast.AsyncFunctionDef | ast.FunctionDef, labels: set | None = None):  # noqa: WPS231
         """Handle a function definition node.
 
@@ -247,17 +315,28 @@ class Visitor(BaseVisitor):  # noqa: WPS338
 
         # handle decorators
         decorators = []
+        overload = False
         if node.decorator_list:
             lineno = node.decorator_list[0].lineno
-            self.in_decorator = True
             for decorator_node in node.decorator_list:
-                decorators.append(Decorator(decorator_node.lineno, decorator_node.end_lineno))  # type: ignore[attr-defined]
-                self.visit(decorator_node)
-            self.in_decorator = False
+                decorator_value = get_value(decorator_node)
+                overload = (
+                    decorator_value == "typing.overload"
+                    or decorator_value == "overload"
+                    and self.current.resolve("overload") == "typing.overload"
+                )
+                decorators.append(
+                    Decorator(
+                        decorator_value,
+                        lineno=decorator_node.lineno,
+                        endlineno=decorator_node.end_lineno,  # type: ignore[attr-defined]
+                    )
+                )
         else:
             lineno = node.lineno
 
-        # TODO: handle member already exist, setter of property
+        labels |= self.decorators_to_labels(decorators)
+        base_property, property_function = self.get_base_property(decorators)
 
         # handle parameters
         parameters = Parameters()
@@ -340,8 +419,23 @@ class Visitor(BaseVisitor):  # noqa: WPS338
             returns=get_annotation(node.returns, parent=self.current),
             decorators=decorators,
             docstring=self._get_docstring(node),
+            runtime=not self.type_guarded,
+            parent=self.current,
         )
-        self.current[node.name] = function
+
+        if overload:
+            self.overloads[function.path].append(function)
+        elif base_property is not None:
+            if property_function == "setter":
+                base_property.setter = function
+                base_property.labels.add("writable")
+            elif property_function == "deleter":
+                base_property.deleter = function
+                base_property.labels.add("deletable")
+        else:
+            self.current[node.name] = function
+            if self.overloads[function.path]:
+                function.overloads = self.overloads[function.path]
 
         function.labels |= labels
 
@@ -381,8 +475,8 @@ class Visitor(BaseVisitor):  # noqa: WPS338
                 alias_path,
                 lineno=node.lineno,
                 endlineno=node.end_lineno,  # type: ignore[attr-defined]
+                runtime=not self.type_guarded,
             )
-        self.generic_visit(node)
 
     def visit_importfrom(self, node: ast.ImportFrom) -> None:  # noqa: WPS231
         """Visit an "import from" node.
@@ -411,8 +505,8 @@ class Visitor(BaseVisitor):  # noqa: WPS338
                 alias_path,  # type: ignore[arg-type]
                 lineno=node.lineno,
                 endlineno=node.end_lineno,  # type: ignore[attr-defined]
+                runtime=not self.type_guarded,
             )
-        self.generic_visit(node)
 
     def handle_attribute(  # noqa: WPS231
         self,
@@ -479,6 +573,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
                 lineno=node.lineno,
                 endlineno=node.end_lineno,  # type: ignore[union-attr]
                 docstring=docstring,
+                runtime=not self.type_guarded,
             )
             attribute.labels |= labels
             parent[name] = attribute
@@ -503,6 +598,20 @@ class Visitor(BaseVisitor):  # noqa: WPS338
         """
         self.handle_attribute(node, get_annotation(node.annotation, parent=self.current))
 
+    def visit_if(self, node: ast.If) -> None:
+        """Visit an "if" node.
+
+        Parameters:
+            node: The node to visit.
+        """
+        if isinstance(node.parent, (ast.Module, ast.ClassDef)):  # type: ignore[attr-defined]
+            with suppress(KeyError):  # unhandled AST nodes
+                condition = get_annotation(node.test, self.current)
+                if str(condition) in {"typing.TYPE_CHECKING", "TYPE_CHECKING"}:
+                    self.type_guarded = True
+        self.generic_visit(node)
+        self.type_guarded = False
+
 
 _patched = False
 
