@@ -161,7 +161,11 @@ def _read_block(docstring: Docstring, offset: int) -> tuple[str, int]:
     return "\n".join(block).rstrip("\n"), new_offset - 1
 
 
-def _read_parameters(docstring: Docstring, offset: int) -> tuple[list[DocstringParameter], int]:  # noqa: WPS231
+def _read_parameters(  # noqa: WPS231
+    docstring: Docstring,
+    offset: int,
+    warn_unknown_params: bool = True,
+) -> tuple[list[DocstringParameter], int]:  # noqa: WPS231
     parameters = []
     annotation: str | Name | Expression | None
 
@@ -202,6 +206,14 @@ def _read_parameters(docstring: Docstring, offset: int) -> tuple[list[DocstringP
         if annotation is None:
             _warn(docstring, line_number, f"No type or annotation for parameter '{name}'")
 
+        if warn_unknown_params and docstring.parent is not None:
+            if name not in docstring.parent.parameters:  # type: ignore[attr-defined]
+                _warn(
+                    docstring,
+                    line_number,
+                    f"Parameter '{name}' does not appear in the parent signature",
+                )
+
         parameters.append(DocstringParameter(name=name, value=default, annotation=annotation, description=description))
 
     return parameters, new_offset
@@ -226,7 +238,7 @@ def _read_other_parameters_section(
     offset: int,
     **options: Any,
 ) -> tuple[DocstringSectionOtherParameters | None, int]:
-    parameters, new_offset = _read_parameters(docstring, offset)
+    parameters, new_offset = _read_parameters(docstring, offset, warn_unknown_params=False)
 
     if parameters:
         return DocstringSectionOtherParameters(parameters), new_offset
@@ -367,7 +379,7 @@ def _read_returns_section(  # noqa: WPS231
                             annotation = return_item
 
             if annotation is None:
-                returned_value = repr(name) or index
+                returned_value = repr(name) if name else index + 1
                 _warn(docstring, line_number, f"No type or annotation for returned value {returned_value}")
 
         returns.append(DocstringReturn(name=name or "", annotation=annotation, description=description))
@@ -418,7 +430,7 @@ def _read_yields_section(  # noqa: WPS231
                         annotation = yield_item
 
             if annotation is None:
-                yielded_value = repr(name) or index
+                yielded_value = repr(name) if name else index + 1
                 _warn(docstring, line_number, f"No type or annotation for yielded value {yielded_value}")
 
         yields.append(DocstringYield(name=name or "", annotation=annotation, description=description))
@@ -464,7 +476,7 @@ def _read_receives_section(  # noqa: WPS231
                         annotation = receives_item
 
         if annotation is None:
-            received_value = repr(name) or index
+            received_value = repr(name) if name else index + 1
             _warn(docstring, line_number, f"No type or annotation for received value {received_value}")
 
         receives.append(DocstringReceive(name=name or "", annotation=annotation, description=description))
